//===========================================================
//=     University of Illinois at Urbana/Champaign          =
//=     Department of Computer Science                      =
//=     Dr. Dan Roth - Cognitive Computation Group          =
//=                                                         =
//=  Project: Snow                                          =
//=                                                         =
//=   Module: Example.cpp                                   =
//=  Version: 2.1                                           =
//=  Authors: Jeff Rosen and Andrew Carlson                 =
//=     Date: xx/xx/99                                      = 
//=                                                         =
//===========================================================

#include "Example.h"
#include "GlobalParams.h"
#include <algorithm>

void Example::Show()
{
    FeatureMap::const_iterator it = features.begin();
    FeatureMap::const_iterator end = features.end();
    while (it != end)
    {
        cout << it->first << '(' << it->second << ')';
        it++;
        if (it != end)
        {
            cout << ", ";
        }
    }

    cout << endl;
}

bool Example::Parse( string& in )
{
    features.clear();

    string work(in);
    FeatureID f;
    char* pDelim, delim;

    do {
        f = strtoul(work.c_str(), &pDelim, 10);
        if (pDelim != work.c_str())
        {
            double strgth = 1.0;
            if (*pDelim == ';')
            {
                strgth = strtod(pDelim + 1, &pDelim);
            }
            
            if (!features.insert(make_pair(f,strgth)).second)
            {
                cerr << "Duplicate feature (" << f << ") found!" << endl;
            }
            // Save delim and update work
            delim = *pDelim;
            work = string(pDelim + 1);
        } else {
            cerr << endl << "Problem found parsing example: '"
                 << in.c_str() << "'" << endl;
            return false;
        }
    } while ((delim == ',') || (delim == ';'));

    return true;    
}

bool Example::Read( ifstream& in)
{
    features.clear();

    FeatureID f;
    char delim;
   
    label = (FeatureID)-1;
    
    // add the fixed feature
    features.insert(make_pair(FIXED_FEATURE_ID,1.0));

    do {
        in >> f;
        
        if (!in.fail())
        {
            in.get(delim);
            double strgth = 1.0;
            if (delim == '(')
            {
                in >> strgth;
                if (in.fail()) goto inerror;
                in.get(delim); // skip )
                in.get(delim); // skip ,
            }
            
            if (!features.insert(make_pair(f,strgth)).second)
            {
                cerr << "Duplicate feature (" << f << ") found!" << endl;
            }
        } else {
inerror:
            if (!in.eof())
            {
                cerr << endl << "Problem found in input file: '"
                     << globalParams.inputFile.c_str()
                     << "'; Ignoring rest of line..." << endl;
                in.clear();
                in.get(delim);
                while (!in.eof() && (delim != '\n'))
                {
                    cerr.put(delim);
                    in.get(delim);
                }
                cerr << endl << endl;
            }
        }
    } while (delim == ',');

    return (!in.eof());
}

bool Example::ReadLabeled( ifstream& in)
{
    features.clear();

    FeatureID f;
    char delim;

    // add the fixed feature
    features.insert(make_pair(FIXED_FEATURE_ID,1.0));

    // read the label and make sure it's a target
    in >> f;
    if (globalParams.targetIds.find(f) != globalParams.targetIds.end())
        {
            label = f;
        }
    else // first feature is not a target-- treat as a regular feature
    {
        features.insert(make_pair(f,1.0)).second;
    }   
    
    in.get(delim);
    
    while (delim == ',') {
        in >> f;
        if (!in.fail())
        {
            in.get(delim);
            double strgth = 1.0;
            if (delim == '(')
            {
                in >> strgth;
                if (in.fail()) goto inerror;
                in.get(delim); // skip )
                in.get(delim); // skip ,
            }
            
            if (!features.insert(make_pair(f,strgth)).second)
            {
                cerr << "Duplicate feature (" << f << ") found!" << endl;
            }
        } else {
inerror:
            if (!in.eof())
            {
                cerr << endl << "Problem found in input file: '"
                     << globalParams.inputFile.c_str()
                     << "'; Ignoring rest of line..." << endl;
                in.clear();
                in.get(delim);
                while (!in.eof() && (delim != '\n'))
                {
                    cerr.put(delim);
                    in.get(delim);
                }
                cerr << endl << endl;
            }
        }
    } 

    return (!in.eof());
}

void Example::Write( ofstream& out )
{
    FeatureMap::const_iterator it = features.begin();
    FeatureMap::const_iterator end = features.end();
    while (it != end)
    {
        out << it->first << ';' << it->second;
        it++;
        if (it != end)
        {
            out << ',';
        }
    }

    out << ':' << endl;
}

